Nrep = 1; % index the results, use 1 for real data.
for rep=Nrep

%%% Initialization

%%initialize random seed
mystream = RandStream('mt19937ar','Seed',sum(10000*rep));
RandStream.setGlobalStream(mystream);

%%creat a folder to store MCMC samples of latent processes for MR algorithm
if(not(exist('EY')))
	mkdir EY
end

% compile the mex function, should include the header and library of the gsl library
%mex MC18_lz_hd.c -I/opt/local/include/ -L/opt/local/lib/ -lgsl -lgslcblas
mex MC18_lz_hd.c -lgsl -lgslcblas


%% define some constants
NEMS0=1;
NR_ALL=[0,1,3,7]; % number of imputation between two observed time points
NNR=size(NR_ALL,2); % number of resolutions
Npara=43; % number of parameters in the dynamic models
NLat=1; % dimension of latent processes

NZ=3; % dimension of observed measurements
NX=7; % dimension of covariates in the dynamic model
NXV=2; % dimension of time varying covariate in the measurement model

MCMAX=6%000; % number of total MCMC sample
Nburn=2%000; % number of Burn-in sample
nthin=1;	% thinning in MCMC
MRMp=1-0.3;	% probability of local samples
indscale=1;	% Not used anymore
alpha=0.1;	% the alpha parameter in the modified Bridge sample
FL=1;		% Is the processes observed (0) of latent (1)
bridgeprop=0.5;	% probability of bridge sample out of the two local samplers

%% file names to be read
FnTimeLen = 'data/NOALL.txt'; % number of observations of each subject, a vector equals, length = number of subject
FnObsLoc = 'data/obsloc_1.txt'; % location of observation of each subject in the initial imputation, a matrix, number of subject * maximum number of time points over subjects
FnMeasurements = 'data/Z_1.txt'; % measurements array, dimension: # of measurements * # of time points at inital imputation * # of subjects
FnTimeDifference = 'data/dt_1.txt'; % time difference between adjacent time points, matrix, (# of time points (initlal imputation) -1) * # of subjects
FnDynCovariates = 'data/X_1.txt'; % time invarant covariate, matrix, # of covariate * # of subjects 
FnCovariates = 'data/XV_1.txt'; % time varying covariate, matrix, # of covariate * # of subjects
FnLatProcessInitialValue = 'data/Y0_1.txt'; % initial values of latent processes # of latent processes * # of time points at inital imputation * # of subjects 


%% read files and extract constants
NO_ALL=dlmread(FnTimeLen);

NObs=numel(NO_ALL);
NO1_ALL=NO_ALL+1;
NO=max(NO_ALL);
NO1=max(NO1_ALL);

obsloc=zeros(NObs,NO1);
fid=fopen(FnObsLoc,'r');
tmp=fscanf(fid,'%f');
fclose(fid);
obsloc(:,:) = reshape(tmp,[NObs,NO1]); 

T=zeros(NO,NObs);
Z=zeros(NZ,NO1,NObs);
X=zeros(NX,NObs);
XV=zeros(NXV,NO,NObs);

fid=fopen(FnMeasurements,'r');
tmp=fscanf(fid,'%f');
fclose(fid);
Z(:,:,:)=reshape(tmp,[NZ,NO1,NObs]);

fid=fopen(FnTimeDifference,'r');
tmp=fscanf(fid,'%f');
fclose(fid);
T(:,:)=reshape(tmp,[NO,NObs]);

fid=fopen(FnDynCovariates,'r');
tmp=fscanf(fid,'%f');
fclose(fid);
X(:,:)=reshape(tmp,[NX,NObs]);

fid=fopen(FnCovariates,'r');
tmp=fscanf(fid,'%f');
fclose(fid);
XV(:,:,:)=reshape(tmp,[NXV,NO,NObs]);


Nrec=MCMAX-Nburn;

%% define initial values and prior hyperparameters

y0=[1];   %not used
upp=[2e10];	% not used upper limit of latent processes
low=[-2e10]; % not used lower limit of latent processes
yrange=[low,upp]; %not used

pararange=[-2e10,2e10;-2e10,2e10;-2e10,2e10;-2e10,2e10;0,2e10;-2e10,2e10;-2e10,2e10;-2e10,2e10;-2e10,2e10;-2e10,2e10];
% not used, range of parameters

% identification conditions, 0 - fixed; 1 - free to be sampled
ILamb=[0;1;1];	% Loading matrix
Imu=[1,1,1];	% measurement mean (Intercept)
SIGI=[1,1,1];	% measurement error variances check?

%Prior hyperparameters of para in measurement equation
PMLamb=[1;1.0;1.0];	% Loading prior mean
PSLamb=[10000;10000;10000]; % Loading prior variance
PMmu=[0;0;0];	% measurement mean prior mean
PSmu=[10000;10000;10000]; % measurement mean prior variance
Pmu=[PMmu,PSmu];
PSIG1=[3;3;3];	% measurement error variances gamma prior 1st hyperparameter
PSIG2=[0.6;0.6;0.6]; % measurement error variances gamma prior 2nd hyperparameter
PSIG=[PSIG1,PSIG2];

%not used Prior hyperparameters of para in dynamic equation
prior_hyper0=zeros(Npara,3); %not used
prior_hyper0(:,1)=0;
prior_hyper0(:,2)=1000000;
prior_hyper0(:,3)=1;
prior_hyper0(Npara,1)=0.0001;
prior_hyper0(Npara,2)=0.0001;
prior_hyper0(Npara,3)=2;

numhyp=0:2:((Npara-1)*2); % not used

%not used, RW-MH tuning parameters of para in structural equation (dynamic parameters) check?
%tun_para=[0.05,0.05,0.03,0.02,0.02,0.02];
tun_para=[0.03,0.03,0.1,0.001,0.001,0.1,0.1,0.1,0.1,0.1]; % not used

Ytun=[0.01]; % not used

%true drift function and diffusion functions
drift_fun=cell(NLat,1);
diffu_fun=cell(NLat,1);

%% Initial values of parameters
para=[1.0,0.0,0.0,0.0,0.0,0.0,0.0,...
      1.0,0.0,0.0,0.0,0.0,0.0,0.0,...
      1.0,0.0,0.0,0.0,0.0,0.0,0.0,...
      1.0,0.0,0.0,0.0,0.0,0.0,0.0,...
      1.0,0.0,0.0,0.0,0.0,0.0,0.0,...
      1.0,0.0,0.0,0.0,0.0,0.0,0.0,...
      1.0];

Lamb=[1;1;1];
SIG=0.1*ones(1,NZ);
mu=[0.0;-0.3;0.3];

Y0=zeros(NLat,NO1,NObs);
fid=fopen(FnLatProcessInitialValue,'r');
tmp=fscanf(fid,'%f');
fclose(fid);
Y0(:,:,:)=reshape(tmp,[NLat,NO1,NObs]);


%% Gibbs sampler starts

% calculate some initial values automatically

jnr=1;

NR=NR_ALL(jnr);

N1=max(max(obsloc));
N=N1-1;


Y=zeros(NLat,N1,NObs);
%Y_p=zeros(N1,1);

for l=1:NObs
    Y(:,obsloc(l,1:NO1_ALL(l)),l) = Y0(:,1:NO1_ALL(l),l);
    for i = 1:NO_ALL(l)
        j = obsloc(l,i)+1;
        while j<obsloc(l,i+1)
            Ybeg = Y(:,obsloc(l,i),l);
            Yend = Y(:,obsloc(l,i+1),l);
            Y(:,j,l)=Ybeg+(Yend-Ybeg)*(j-obsloc(l,i))/(obsloc(l,i+1)-obsloc(l,i));
            j = j+1;
        end
    end
end
Ytmp = reshape(Y,[N1,NObs]);


XV1 = zeros(NXV,N1-1,NObs);

dt_NR=zeros(N,NObs);
for l=1:NObs
    for i = 1:NO_ALL(l)
    j = obsloc(l,i);
        while j<obsloc(l,i+1)
            NRtmp = obsloc(l,i+1) - obsloc(l,i);
            dt_NR(j,l)=T(i,l)/(NRtmp);
            XV1(:,j,l) = XV(:,i,l);      
            j = j+1;
        end
    end
end
Xtmp = reshape(XV1(1,:,:),[N1-1,NObs]);
obsloc1 = obsloc - 1;

% store data for checking

fid=fopen('Z_r.txt','w');
fprintf(fid,'%f\n',Z);
fclose(fid);

fid=fopen('T_r.txt','w');
fprintf(fid,'%f\n',T);
fclose(fid);

fid=fopen('Y0_r.txt','w');
fprintf(fid,'%f\n',Y0);
fclose(fid);

fid=fopen('X_r.txt','w');
fprintf(fid,'%f\n',X);
fclose(fid);

fid=fopen('XV_r.txt','w');
fprintf(fid,'%f\n',XV1);
fclose(fid);

% form cells to be passed to mex, automatically

In_allscalar=cell(16,1);
In_allscalar{1}=MCMAX;
In_allscalar{2}=Nburn;
In_allscalar{3}=NObs;
In_allscalar{4}=NLat;
In_allscalar{5}=NO;
In_allscalar{6}=Npara;
In_allscalar{7}=MRMp;
In_allscalar{8}=numel(NR_ALL);
In_allscalar{9}=NZ;
In_allscalar{10}=FL;
In_allscalar{11}=indscale;
In_allscalar{12}=alpha;
In_allscalar{13}=bridgeprop;
In_allscalar{14}=nthin;
In_allscalar{15}=NX;
In_allscalar{16}=NXV;

In_allarray=cell(26,1);
In_allarray{1}=Y;
In_allarray{2}=para;
In_allarray{3}=dt_NR;
In_allarray{4}=yrange;
In_allarray{5}=pararange;
In_allarray{6}=tun_para;
In_allarray{7}=prior_hyper0;
In_allarray{8}=numhyp;
In_allarray{9}=NR_ALL;
In_allarray{10}=Lamb;%=[1,0,0;1,0,0;1,0,0;0,1,0;0,1,0;0,1,0;0,0,1;0,0,1;0,0,1];
In_allarray{11}=ILamb;%=[0,0,0;1,0,0;1,0,0;0,0,0;0,1,0;0,1,0;0,0,0;0,0,1;0,0,1];
In_allarray{12}=mu;%=[1,1;1;1;1;1;1;1;1];
In_allarray{13}=Imu;%=[1;11;1;1;1;1;1;1];
In_allarray{14}=SIG;%=0.3*ones(1,NZO);
In_allarray{15}=SIGI;%=[1,1,1,1,1,1,1,1,1];
In_allarray{16}=Z;
In_allarray{17}=Pmu;
In_allarray{18}=PMLamb;
In_allarray{19}=PSLamb;
In_allarray{20}=PSIG;
In_allarray{21}=Y0;
In_allarray{22}=NO_ALL;
In_allarray{23}=Ytun;
In_allarray{24}=X;
In_allarray{25}=obsloc1;
In_allarray{26}=XV1;


mystream = RandStream('mt19937ar','Seed',sum(10000*rep));
RandStream.setGlobalStream(mystream);

% run the mex function 

tic
MC18_lz_hd(In_allscalar,In_allarray);
toc

for i = 1:numel(NR)
copyfile(strcat('Epara',num2str(i-1),'.txt'),strcat('Epara',num2str(i-1),'_',num2str(rep),'.txt'));
copyfile(strcat('ESIG',num2str(i-1),'.txt'),strcat('ESIG',num2str(i-1),'_',num2str(rep),'.txt'));
copyfile(strcat('Emu',num2str(i-1),'.txt'),strcat('Emu',num2str(i-1),'_',num2str(rep),'.txt'));
copyfile(strcat('ELamb',num2str(i-1),'.txt'),strcat('ELamb',num2str(i-1),'_',num2str(rep),'.txt'));
end

end

%MR_summary('Epara',NNR,Nrep,Nrec,Npara);
%MR_summary('ESIG',NNR,Nrep,Nrec,NZ);
%MR_summary('Emu',NNR,Nrep,Nrec,NZ);
%MR_summary('ELamb',NNR,Nrep,Nrec,2);

%fid=fopen('Epara0.txt')
%A = fscanf(fid, '%g %g %g', [3 inf]);
%fclose(fid)
%A=A'
%mean(A)
%rmdir('EY');

